@@ -5,7 +5,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 use Statistics::Covid::Utils;
 use Statistics::Covid::Datum;
@@ -71,10 +71,13 @@ sub	new {
 				'debug' => $debug
 			});
 			if( ! defined $providerObj ){ warn "error, call to $pn->new() has failed."; return undef }
-			$providers{$pn} = $providerObj;
+			# the key to the providers can be the full package or just this bit
+			# we prefer this bit (e.g. World::JHU)
+			#$providers{$pn} = $providerObj;
+			$providers{$aprovider} = $providerObj;
 			if( $debug > 0 ){ warn "provider added '$pn':\n".$providerObj->toString() }
 		}
-		$self->{'p'}->{'provider-objs'} = \%providers;
+		$self->{'p'}->{'provider-objs'} = {%providers};
 	} else { warn "error, 'providers' was not specified, a list of provider names, e.g. 'UK::GOVUK' and/or 'World::JHU' must be provided."; return undef }
 
 	for(keys %$params){
@@ -87,6 +90,14 @@ sub	new {
 
 	return $self
 }
+sub	DESTROY {
+	# disconnect just in case, usually this is not required
+	my $self = $_[0];
+	if( defined $self->{'p'}->{'datum-io'} ){
+		$self->{'p'}->{'datum-io'}->db_disconnect();
+		$self->{'p'}->{'datum-io'} = undef;
+	}
+}
 # fetch data from the providers and optionally save to file and/or db
 # returns undef on failure
 # returns the items fetched (that's datum objects) as an arrayref (which can also be empty)
@@ -128,6 +139,15 @@ sub	fetch_and_store {
 	# returns an arrayref of all the Datums JUST fetched (after being converted from raw data to objects)
 	return \@retObjs
 }
+# a shortcut to saving an arrayref of Datum Objects to our db
+# returns a hashref of statistics on what happened with the insert
+# see L<Statistics::Covid::IO::Base::db_insert_bulk>() for details
+# returns undef on failure
+sub	db_datums_insert_bulk { return $_[0]->{'p'}->{'datum-io'}->db_insert_bulk($_[1]) }
+# a shortcut to gettting the count of the Datum table,
+# returns -1 on failure
+sub	db_datums_count { return $_[0]->{'p'}->{'datum-io'}->db_count() }
+
 # load datums from DB into our own internal storage (appending to whatever we already may have stored)
 # use clear() to empty thats storage.
 # input params are exactly what Statistics::Covid::IO::Base::db_select($params) takes
@@ -164,6 +184,115 @@ sub	select_datums_from_db_for_location {
 	if( ! defined $results ){ warn "error, call to ".'select_datums_from_db()'." has failed."; return undef }
 	return $results
 }
+# read data from data file (original data as fetched by the scrapper)
+# given at least a provider string in the input params hash
+#         as $params->{'provider'} = 'XYZ'
+# in which case all data found in XYZ's datafilesdir will be read
+# Now, the files to read will be specified in the
+#            $params->{'basename'} = 'ABC' | ['ABC', '123', ...]
+# which again can be a scalar if it's a single basename
+# or an arrayref for one or more.
+# All the basenames will apply to the provider specified.
+# Each basename will be used to construct the exact data-file name(s)
+# for the specified provider.
+# The specified provider string id must correspond to a provider object
+# already created and loaded during construction via the 'providers' param
+# returns undef on failure
+# returns an array of L<Statistics::Covid::Datum> Objects on success
+sub	read_data_from_file {
+	my $self = $_[0];
+	my $params = $_[1];
+
+	my $debug = $self->debug();
+
+	my $providerstr;
+	if( ! exists($params->{'provider'}) || ! defined($providerstr=$params->{'provider'}) ){ warn "error, 'provider' was not specified."; return undef }
+	my $providerObj = $self->providers($providerstr);
+	if( ! defined $providerObj ){ warn "provider does not exist in my list, you may need to load it if indeed the name is correct: '$providerstr'."; return undef }
+
+	# optional list of basenames of data
+	# if this is missing then all files in the datafilesdir() of the provider specified
+	# will be loaded
+	my (@basenames, $m);
+	if( exists($params->{'basename'}) && defined($m=$params->{'basename'}) ){
+		# basename was specified, we expect an arrayref of basenames or a single basename
+		my $r = ref($m);
+		if( $r eq '' ){ @basenames = ($m) }
+		elsif( $r eq 'ARRAY' ){ @basenames = @{$m} }
+		else { warn "error, expected scalar string or arrayref for input 'basename' but got ".$r; return undef }
+	} else {
+		# no basenames specified, find them
+		my $datadir = $providerObj->datafilesdir();
+		# TODO : remove this check when stable
+		if( ! defined($datadir) ){ die "something wrong here, datafilesdir() is not specified for provider '$providerstr'." }
+		my $jsonfiles = Statistics::Covid::Utils::find_files($datadir, qr/\.json$/i);
+		@basenames = map { s/\.((data)|(meta))\.json$//; $_ } @$jsonfiles;
+	}
+	if( scalar(@basenames) == 0 ){
+		warn "no files were found or specified for provider '$$providerstr'.";
+		return () # not a failure
+	}
+	my @ret;
+	for my $abasename (@basenames){
+		my $datas = $providerObj->load_fetched_data_from_localfile($abasename);
+		if( ! defined $datas ){ warn "error, call to ".'load_fetched_data_from_localfile()'." has failed for provider '$providerstr' and data-file basename '$abasename'."; return undef }
+		# convert datas to datums
+		my $datumObjs = $providerObj->create_Datums_from_fetched_data($datas);
+		if( $debug > 0 ){ warn "read ".scalar(@$datumObjs)." items from basename '$abasename'.\n"; }
+		if( ! defined $datumObjs ){ warn "error, call to ".'create_Datums_from_fetched_data()'." has failed for provider '$providerstr' and data-file basename '$abasename'."; return undef }
+		push @ret, @$datumObjs
+	}
+	if( $debug > 0 ){ warn "read ".scalar(@ret)." items in total for provider '$providerstr'." }
+	return \@ret # success
+}
+# read data from data files given
+# an input hashref of $params->{'what'}={providerID => [list-of-basenames]}
+# returns a hashref {providerID => $datumObjs}
+# or undef on failure, for more information see L<read_data_from_file()>
+sub	read_data_from_files {
+	my $self = $_[0];
+	my $params = $_[1];
+	my $inp;
+
+	my @providerstrs;
+	if( ! exists($params->{'what'}) || ! defined($params->{'what'}) ){
+		# nothing was provided in the input, we use ALL our providers loaded during construction
+		@providerstrs = keys %{$self->providers()};
+	} else {
+		# something was given at input
+		$inp = $params->{'what'};
+		if( ref($inp) eq 'HASH' ){
+			# this is a hash of {providerID => [list-of-basenames]}
+			my %ret;
+			for my $aproviderstr (sort keys %$inp){
+				my $da = $self->read_data_from_file({
+					'basename' => $inp->{$aproviderstr}
+				});
+				if( ! defined $da ){ warn "error, call to read_data_from_file() has failed for provider '$aproviderstr'."; return undef }
+				$ret{$aproviderstr} = $da;
+			}
+			return \%ret
+		} elsif( ref($inp) eq 'ARRAY' ){
+			# this is an array of provider strings,
+			# all files from the data dir of this provider will be loaded
+			@providerstrs = @$inp;
+		} elsif( ref($inp) eq '' ){
+			# this is just a lone provider string
+			@providerstrs = ($inp);
+		}
+	}
+	# Here we have a list of providerstrs only
+	# So, we will let read_data_from_file() find data files in our datafilesdir()
+	my %ret;
+	foreach my $aproviderstr (@providerstrs){
+		my $da = $self->read_data_from_file({
+			'provider' => $aproviderstr
+		});
+		if( ! defined($da) ){ warn "error, call to read_data_from_file() has failed for the provider '$aproviderstr'."; return undef }
+		$ret{$aproviderstr} = $da;
+	}
+	return \%ret
+}
 # read the Version table from DB which holds the db-version which is useful
 # for migrating to newer db-versions when upgrades happen.
 # ideally each upgrade should have a migration script for migrating from
@@ -245,7 +374,23 @@ sub     debug {
 	return $self->{'debug'}
 }
 sub     dbparams { return $_[0]->config()->{'dbparams'} }
-sub	providers { return $_[0]->{'p'}->{'provider-objs'} }
+# returns the hashref of providers (id=>obj) if no input
+# provider string id is provided
+# else checks to see if a provider can be matched from our list
+# and if it does, it returns the provider obj
+# if not found returns undef (so undef can happen only
+# if a provider id is specified at input)
+sub	providers {
+	my $self = $_[0];
+	my $pstr = $_[1];
+	if( defined $pstr ){
+		# return the exact provider if the pstr matches an id from our providers
+		return($self->{'p'}->{'provider-objs'}->{$pstr})
+			if exists($self->{'p'}->{'provider-objs'}->{$pstr});
+		return undef # id given is not in our list
+	}
+	return $self->{'p'}->{'provider-objs'} # that's the hash with the providers
+}
 sub     config {
 	my $self = $_[0];
 	my $m = $_[1];
@@ -319,7 +464,7 @@ Statistics::Covid - Fetch, store in DB, retrieve and analyse Covid-19 statistics
 
 =head1 VERSION
 
-Version 0.20
+Version 0.21
 
 =head1 DESCRIPTION
 
@@ -516,9 +661,17 @@ optional and will not be run unless explicitly stated via
 C<make bench>
 
 These tests do not hit the online data providers at all. And they
-should not. They only time the creation of objects and insertion
+should not, see ADDITIONAL TESTING for more information on this.
+They only time the creation of objects and insertion
 to the database.
 
+=head1 ADDITIONAL TESTING
+
+Testing the DataProviders is not done because it requires
+network access and hits on the providers which is not fair.
+However, there are targets in the Makefile for initiating
+the "network" tests by doing C<make network> .
+
 =head1 CAVEATS
 
 This module has been put together very quickly and under pressure.
@@ -530,6 +683,8 @@ to use the data previously collected and stored.
 B<Support for MySQL is totally untested>. Please use SQLite for now
 or test the MySQL interface.
 
+B<Support for Postgres has been somehow missed but is underway!>.
+
 =head1 BUGS
 
 This module has been put together very quickly and under pressure.
