@@ -10,7 +10,7 @@ use File::Copy;
 
 use Statistics::Covid::Utils;
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 sub	new {
 	my (
@@ -265,7 +265,7 @@ sub     db_connect {
 	my $dbiparams = {RaiseError => 1, PrintError => 0};
 	if( exists $dbparams->{'connect-params'} and defined $dbparams->{'connect-params'} ){ $dbiparams = {%$dbiparams, %{$dbparams->{'connect-params'}}} }
 	my $schemaHandle = Statistics::Covid::Schema->connect($dsn, "", "", $dbparams);
-	if( ! defined $schemaHandle or $schemaHandle->storage==1 ){ warn 'Statistics::Covid::Schema->connect('.$dsn.') has failed: '.$DBI::errstr; return undef }
+	if( ! defined $schemaHandle or $schemaHandle->storage==1 ){ warn 'Statistics::Covid::Schema->connect('.$dsn.') has failed.'; return undef }
 	my $debug = $self->debug();
 	$schemaHandle->storage->debug($debug);
 	my $m = $self->logfilename();
@@ -284,9 +284,24 @@ sub     db_connect {
 	if( $debug > 0 ){ warn "db_connect() : connected to '$dsn'." }
 	return $schemaHandle
 }
+# erases all rows in the given table of the database
+# it returns the number of rows deleted (can be zero) on success
+# or -1 on failure
+sub	db_clear { return $_[0]->db_delete_rows() }
+
+# deletes those rows in DB which match the L<SQL::Abstract> search
+# conditions and attributes, optionally specified in the input params
+# C<$params->{'conditions'}> and C<$params->{'attributes'}>
+# WARNING: If no conditions and/or attributes are specified ALL rows in DB will be deleted.
+# In the latter case one may call C<db_clear()> as a mnemonic shortcut.
+# it returns the number of rows deleted (can be zero) on success
+# or -1 on failure
 sub	db_delete_rows {
 	my $self = $_[0];
 	my $params = $_[1];
+
+	if( ! $self->db_is_connected() ){ warn "error, not connected to any database."; return -1 }
+
 	$params = {} unless $params;
 	# params optionally contains 'conditions' something like:
 #  my $rs = $schema->resultset('Album')->search({
@@ -315,10 +330,9 @@ sub	db_delete_rows {
 	my $debug = (exists $params->{'debug'} and defined $params->{'debug'}) ? $params->{'debug'} : $self->debug();
 
 	my $dbparams = $self->dbparams();
-	if( ! exists $dbparams->{'schemah'} or ! defined $dbparams->{'schemah'} ){ warn "error, not connected to db"; return 0 }
 	my $schemah = $dbparams->{'schemahs'};
 	my $RS = $dbparams->{'schemah'}->resultset($self->dual_package_vars()->{'tablename'});
-	if( ! defined $RS ){ warn "error, failed to create a resultset object."; return 0 }
+	if( ! defined $RS ){ warn "error, failed to create a resultset object."; return -1 }
 
 	my $searcher = $RS->search($conditions, $attributes);
 	if( $debug > 0 ){ warn "deleting all rows found with ".${$searcher->as_query()}->[0]."\n" }
@@ -336,6 +350,9 @@ sub	db_delete_rows {
 sub	db_insert_bulk {
 	my $self = $_[0];
 	my $arrayOfobjToInsert = $_[1]; # arrayref with object(s) to insert
+
+	if( ! $self->db_is_connected() ){ warn "error, not connected to any database."; return undef }
+
 	my $debug = $self->debug();
 
 	my $ret = 1;
@@ -353,7 +370,7 @@ sub	db_insert_bulk {
 		$pk = $anobj->primary_key();
 		if( exists $objs_by_pk{$pk} ){
 			$anotherobj = $objs_by_pk{$pk};
-			warn "found duplicate records within the array of objects to return:\n".$anotherobj->toString()."\nand\n".$anobj->toString()."\n";
+			if( $debug > 0 ){ warn "found duplicate records within the array of objects to return:\n".$anotherobj->toString()."\nand\n".$anobj->toString()."\n" }
 			$hasduplicates++;
 			if( $anobj->newer_than($anotherobj) == 1 ){ $objs_by_pk{$pk} = $anobj }
 		} else { $objs_by_pk{$pk} = $anobj }
@@ -417,11 +434,13 @@ sub	db_insert_bulk {
 sub	db_insert {
 	my $self = $_[0];
 	my $objToInsert = $_[1];
+
+	if( ! $self->db_is_connected() ){ warn "error, not connected to any database."; return 0 }
+
 	my $debug = $self->debug();
 
 	my $dbparams = $self->dbparams();
 
-	if( ! exists $dbparams->{'schemah'} or ! defined $dbparams->{'schemah'} ){ warn "error, not connected to db"; return 0 }
 	my $replace_strategy = 'ignore';
 	if( exists $dbparams->{'replace-existing-db-record'} and defined($dbparams->{'replace-existing-db-record'}) ){
 		$replace_strategy = $dbparams->{'replace-existing-db-record'};
@@ -487,6 +506,9 @@ sub	db_insert {
 sub	db_count {
 	my $self = $_[0];
 	my $params = $_[1]; # optional
+
+	if( ! $self->db_is_connected() ){ warn "error, not connected to any database."; return -1 }
+
 	$params = {} unless $params;
 	# params optionally contains 'conditions' something like:
 #  my $rs = $schema->resultset('Album')->search({
@@ -515,7 +537,6 @@ sub	db_count {
 	my $debug = (exists $params->{'debug'} and defined $params->{'debug'}) ? $params->{'debug'} : $self->debug();
 
 	my $dbparams = $self->dbparams();
-	if( ! exists $dbparams->{'schemah'} or ! defined $dbparams->{'schemah'} ){ warn "error, not connected to db"; return -1 }
 	my $schemah = $dbparams->{'schemah'};
 	my $RS = $dbparams->{'schemah'}->resultset($self->dual_package_vars()->{'tablename'});
 	if( ! defined $RS ){ warn "error, failed to create a resultset object."; return -1 }
@@ -532,6 +553,9 @@ sub	db_count {
 sub	db_select {
 	my $self = $_[0];
 	my $params = $_[1];
+
+	if( ! $self->db_is_connected() ){ warn "error, not connected to any database."; return undef }
+
 	$params = {} unless $params;
 	# params optionally contains 'conditions' something like:
 #  my $rs = $schema->resultset('Album')->search({
@@ -560,10 +584,9 @@ sub	db_select {
 	my $debug = (exists $params->{'debug'} and defined $params->{'debug'}) ? $params->{'debug'} : $self->debug();
 
 	my $dbparams = $self->dbparams();
-	if( ! exists $dbparams->{'schemah'} or ! defined $dbparams->{'schemah'} ){ warn "error, not connected to db"; return 0 }
 	my $schemah = $dbparams->{'schemah'};
 	my $RS = $dbparams->{'schemah'}->resultset($self->dual_package_vars()->{'tablename'});
-	if( ! defined $RS ){ warn "error, failed to create a resultset object."; return 0 }
+	if( ! defined $RS ){ warn "error, failed to create a resultset object."; return undef }
 
 	my $searcher = $RS->search($conditions, $attributes);
 	if( $debug > 0 ){ warn "searching with ".${$searcher->as_query()}->[0]."\n" }
@@ -578,10 +601,12 @@ sub	db_select {
 	}
 	return \@results # returns an array of Datum objects, can be empty.
 }
+# disconnects from DB if it is already connected else ignored
+# it returns 1 in any case.
 sub	db_disconnect {
 	my $self = $_[0];
 	my $dbparams = $self->dbparams();
-	if( exists $dbparams->{'schemah'} and defined $dbparams->{'schemah'} ){
+	if( $self->db_is_connected() ){
 		$dbparams->{'schemah'}->storage()->disconnect();
 		$dbparams->{'schemah'} = undef;
 		my $m = $self->logfilename();
